<?php

namespace PyvioPaySdk\lib;

use PyvioPaySdk\config\MerConfig;
use Exception;

class PyvioPay extends PyvioPayRequest
{
    # 生产环境接口基础地址
    const BASE_API_URL = "https://api.pyvio.com";

    # 沙箱环境接口基础地址
    const BASE_API_SANDBOX_URL = "https://sandbox-api.pyvio.com";

    # 调试模式
    public static $isDebug;
    # 生产模式
    public static $isProdMode;
    # 日志路径
    public static $logDir = "";

    # 商户配置信息
    private static $merConfigMap = array();

    public static function init($config_info, $is_object = false, $merchantKey = 'default')
    {

        if (empty($config_info)) {
            try {
                throw new Exception('缺少SDK配置信息');
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        }

        if ($is_object) {
            $config_obj = $config_info;
        } else {
            if (!file_exists($config_info)) {
                try {
                    throw new Exception('SDK配置文件不存在');
                } catch (Exception $e) {
                    echo $e->getMessage();
                }
            }
            $cfg_file_str = file_get_contents($config_info);
            $config_obj = json_decode($cfg_file_str, true);

            // 处理 json 文件格式异常
            if (!$config_obj) {
                try {
                    throw new Exception('SDK配置文件格式异常');
                } catch (Exception $e) {
                    echo $e->getMessage();
                }
            }
        }

        // 是否调试模式
        self::$isDebug = defined("DEBUG") && DEBUG;
        // 日志路径
        self::$logDir = defined("DEBUG") ? LOG : dirname(__FILE__)."/log";
        // 是否生产模式
        self::$isProdMode = defined("PROD_MODE") && PROD_MODE;

        $merConfig = new MerConfig();
        $merConfig->app_id = isset($config_obj['app_id']) ? $config_obj['app_id'] : '';
        $merConfig->app_secret = isset($config_obj['app_secret']) ? $config_obj['app_secret'] : '';
        $merConfig->rsa_private_key = isset($config_obj['rsa_private_key']) ? $config_obj['rsa_private_key'] : '';
        $merConfig->rsa_pyvio_public_key = isset($config_obj['rsa_pyvio_public_key']) ? $config_obj['rsa_pyvio_public_key'] : '';

        self::addMerConfig($merchantKey, $merConfig);
    }

    protected function filterEmptyData($req_params)
    {
        $params = array_filter($req_params, function ($v) {
            if (!empty($v) || $v == '0') {
                return true;
            }
            return false;
        });

        $keysToReplace = [
            'is_b2b'          => 'is_B2B',
            'three_dsecure'   => 'three_d_secure',
            'id_card_front'   => 'idCard_front',
            'id_card_back'    => 'idCard_back',
            'id_card_type'    => 'idCard_type',
            'id_card_hand'    => 'idCard_hand',
            'id_card_country' => 'idCard_country',
        ];

        foreach ($keysToReplace as $oldKey => $newKey) {
            if (array_key_exists($oldKey, $params)) {
                $params[$newKey] = $params[$oldKey];
                unset($params[$oldKey]);
            }
        }

        return $params;
    }

    public static function writeLog($message, $level = "INFO")
    {
        if (self::$isDebug) {
            if (!is_dir(self::$logDir)) {
                mkdir(self::$logDir, 0777, true);
            }

            $log_file = self::$logDir."/pyviopay_".date("Ymd").".log";
            $server_addr = "127.0.0.1";
            if (isset($_SERVER["REMOTE_ADDR"])) {
                $server_addr = $_SERVER["REMOTE_ADDR"];
            }
            $message_format = "[".$level."] [".gmdate("Y-m-d\TH:i:s\Z")."] ".$server_addr." ".$message."\n";
            $fp = fopen($log_file, "a+");
            fwrite($fp, $message_format);
            fclose($fp);
        }
    }

    /**
     * 根据功能编码获取实际请求路径
     * @param $funcCode
     * @return string $req_url
     */
    public function requestUrl($funcCode)
    {
        // 判断请求环境确定接口域名地址
        $req_url = self::$isProdMode ? self::BASE_API_URL : self::BASE_API_SANDBOX_URL;

        // 如果末尾不是/结尾，则拼接/在最后
        if (!PyvioPayTools::endWith($req_url, "/")) {
            $req_url = $req_url."/";
        }

        # 根据功能编码获取实际接口请求地址
        $actualUrl = str_replace(".", "/", $funcCode);
        # 拼接得到完整请求地址
        return $req_url.$actualUrl;
    }

    /**
     * @throws Exception
     */
    public function post($funcCode, $params, $merchantKey)
    {
        # 获取商户配置信息
        $merchantConfig = self::getConfig($merchantKey);
        if (empty($merchantConfig)) {
            throw new Exception('系统公私密钥配置不存在');
        }

        # 根据功能编码获取实际请求路径
        $req_url = $this->requestUrl($funcCode);
        $request_params = $this->filterEmptyData($params);

        // 实例化 http 请求执行
        $request = new PyvioPayRequest();
        return $request->curlRequest($merchantConfig, $req_url, 'POST', $request_params);
    }

    /**
     * @throws Exception
     */
    public function get($funcCode, $params, $merchantKey)
    {
        # 获取商户配置信息
        $merchantConfig = $this->getConfig($merchantKey);
        if (empty($merchantConfig)) {
            throw new Exception('系统公私密钥配置不存在');
        }

        # 根据功能编码获取实际请求路径
        $req_url = $this->requestUrl($funcCode);
        $request_params = $this->filterEmptyData($params);

        if (isset($request_params['headers'])) unset($request_params['headers']);

        // 实例化 http 请求执行
        $request = new PyvioPayRequest();
        $url = $req_url."?".http_build_query($request_params);
        return $request->curlRequest($merchantConfig, $url, 'GET', $params);
    }

    public static function addMerConfig($merchantKey, MerConfig $merConfig)
    {
        self::$merConfigMap[$merchantKey] = $merConfig;
    }

    public static function getConfig($merChantKey = "default")
    {
        return array_key_exists($merChantKey, self::$merConfigMap) ? self::$merConfigMap[$merChantKey] : [];
    }

}
